package main

import (
	"context"
	"errors"
	"net"
	"net/http"
	"os"
	"slices"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
)

var Resolver *net.Resolver

func LookupIP(c *gin.Context) {
	var err error
	var ips []net.IP
	q := c.Param("q")
	ips, err = Resolver.LookupIP(context.Background(), "ip4", q)
	c.JSON(http.StatusOK, gin.H{"ips": ips, "err": err})
}

type RBLLookupIPRequest struct {
	RBLServer string `json:"rblserver"`
}

func RBLLookupIP(c *gin.Context) {
	var err error
	var dnsErr *net.DNSError
	var listed string

	ip := c.Param("ip")
	ipAddress := net.ParseIP(ip)

	// check if valid
	if ipAddress.To4() == nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": "invalid IP address"})
		return
	}

	// get requested data
	var req RBLLookupIPRequest
	err = c.BindJSON(&req)
	if err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": "invalid request body"})
		return
	}

	// reverse the ip
	s := strings.Split(ipAddress.String(), ".")
	slices.Reverse(s)

	// and add the rbl server
	s = append(s, req.RBLServer)

	// final query
	q := strings.Join(s, ".")
	_, err = Resolver.LookupIP(context.Background(), "ip4", q)

	listed = "listed"
	if err == nil {
		c.JSON(http.StatusOK, gin.H{"status": listed, "query": q})
		return
	}

	if !errors.As(err, &dnsErr) {
		c.JSON(http.StatusRequestTimeout, gin.H{"error": err.Error()})
		return
	}

	if dnsErr.IsTimeout {
		c.JSON(http.StatusRequestTimeout, gin.H{"error": "timeout", "query": q})
		return
	}

	if dnsErr.IsTemporary {
		c.JSON(http.StatusRequestTimeout, gin.H{"error": "temporary error", "query": q})
		return
	}

	if !dnsErr.IsNotFound {
		c.JSON(http.StatusRequestTimeout, gin.H{"error": "temporary error", "query": q})
		return
	}

	if dnsErr.IsNotFound {
		listed = "not listed"
	}

	c.JSON(http.StatusOK, gin.H{"status": listed, "query": q})
}

func initResolver() {
	DNS_SERVER := os.Getenv("DNS_SERVER")
	DNS_PORT := os.Getenv("DNS_PORT")

	if DNS_SERVER == "" {
		DNS_SERVER = "127.0.0.1"
	}

	if DNS_PORT == "" {
		DNS_PORT = "53"
	}

	Resolver = &net.Resolver{
		PreferGo: false,
		Dial: func(ctx context.Context, network, address string) (net.Conn, error) {
			d := net.Dialer{
				Timeout: time.Millisecond * time.Duration(10000),
			}
			return d.DialContext(ctx, network, DNS_SERVER+":"+DNS_PORT)
		},
	}
}

var Router *gin.Engine

func init() {
	initResolver()
	Router = gin.Default()

	accesslog := os.Getenv("ACCESSLOG")
	if accesslog == "false" {
		Router = gin.New()
	}

	Router.GET("/ip4/:q", LookupIP)
	Router.POST("/rbl/:ip", RBLLookupIP)
}

func main() {
	Router.Run(":8000")
}
